@@ -12,8 +12,9 @@ Provides
 
 """
 
-__version__ = "0.4.0"
+__version__ = "0.5.1"
 
+from . import chem
 from . import fileio
 from . import container
 from . import utils
@@ -23,12 +24,16 @@ from . import lcms
 from . import simulation
 from . import raw_data_utils
 from . import _mzml
+from . import _build_data_matrix
 from . import correspondence
+from . import fill_missing
+from . import consensus_annotation
 from .container import DataContainer
 from .fileio import MSData
 from .lcms import Chromatogram, MSSpectrum
 from .assay import Assay
 from .raw_data_utils import *
+from . import annotation
 
 utils.create_tidyms_dir()
 SETTINGS = utils.get_settings()
