@@ -20,6 +20,7 @@ get_find_centroid_params
 """
 
 import bokeh.plotting
+import json
 import numpy as np
 from scipy.ndimage import gaussian_filter1d
 from typing import Dict, List, Optional, Tuple
@@ -29,6 +30,7 @@ from scipy.integrate import cumtrapz
 from . import peaks
 from . import _plot_bokeh
 from . import _constants as c
+from .utils import array1d_to_str, str_to_array1d
 
 
 class MSSpectrum:
@@ -131,7 +133,7 @@ class MSSpectrum:
         fig_params: Optional[dict] = None,
         line_params: Optional[dict] = None,
         show: bool = True
-    ) -> bokeh.plotting.Figure:     # pragma: no cover
+    ) -> bokeh.plotting.figure:     # pragma: no cover
         """
         Plot the spectrum using Bokeh.
 
@@ -140,13 +142,13 @@ class MSSpectrum:
         fig_params : dict or None, default=None
             key-value parameters to pass to ``bokeh.plotting.figure``.
         line_params : dict, or None, default=None
-            key-value parameters to pass to ``bokeh.plotting.Figure.line``.
+            key-value parameters to pass to ``bokeh.plotting.figure.line``.
         show : bool, default=True
             If True calls ``bokeh.plotting.show`` on the Figure.
 
         Returns
         -------
-        bokeh.plotting.Figure
+        bokeh.plotting.figure
 
         """
         default_fig_params = _plot_bokeh.get_spectrum_figure_params()
@@ -155,7 +157,7 @@ class MSSpectrum:
             fig_params = default_fig_params
         else:
             fig_params = default_fig_params
-        fig = bokeh.plotting.Figure(**fig_params)
+        fig = bokeh.plotting.figure(**fig_params)
 
         if self.is_centroid:
             plotter = _plot_bokeh.add_stems
@@ -200,9 +202,9 @@ class Roi:
 
     def plot(
         self,
-        figure: Optional[bokeh.plotting.Figure] = None,
+        figure: Optional[bokeh.plotting.figure] = None,
         show: bool = True
-    ) -> bokeh.plotting.Figure:     # pragma: no cover
+    ) -> bokeh.plotting.figure:     # pragma: no cover
         raise NotImplementedError
 
     def extract_features(self, **kwargs) -> List["Feature"]:
@@ -266,15 +268,17 @@ class Roi:
     def get_default_filters(self) -> Dict[str, float]:
         raise NotImplementedError
 
-    def fill_nan(self, fill_value: Optional[float] = None):
+    def fill_nan(self, **kwargs):
         """
-        Fill missing intensity values using linear interpolation.
+        Fill missing values. Missing m/z values are filled using the mean m/z
+        of the ROI. Missing intensity values are filled using linear
+        interpolation. Missing values on the boundaries are filled by
+        extrapolation. Negative values are set to 0.
 
         Parameters
         ----------
-        fill_value : float or None
-            Missing intensity values are replaced with this value. If None,
-            values are filled using linear interpolation.
+        kwargs:
+            Parameters to pass to :func:`scipy.interpolate.interp1d`
 
         """
 
@@ -282,21 +286,29 @@ class Roi:
         # of zero. This prevents errors in the interpolation and makes peak
         # picking work better.
 
-        if np.isnan(self.spint[0]):
-            self.spint[0] = 0
-        if np.isnan(self.spint[-1]):
-            self.spint[-1] = 0
-
         missing = np.isnan(self.spint)
-        if fill_value is None:
-            interpolator = interp1d(self.time[~missing], self.spint[~missing])
+        if missing.any():
+            interpolator = interp1d(
+                self.time[~missing],
+                self.spint[~missing],
+                assume_sorted=True,
+                **kwargs
+            )
+            sp_max = np.nanmax(self.spint)
+            sp_min = np.nanmin(self.spint)
             self.spint[missing] = interpolator(self.time[missing])
-        else:
-            self.spint[missing] = fill_value
-
+            # bound extrapolated values to max and min observed values
+            self.spint = np.maximum(self.spint, sp_min)
+            self.spint = np.minimum(self.spint, sp_max)
         if isinstance(self.mz, np.ndarray):
-            mz_mean = np.nanmean(self.mz)
-            self.mz[missing] = mz_mean
+            self.mz[missing] = np.nanmean(self.mz)
+
+    @staticmethod
+    def from_json(s: str) -> "Roi":
+        raise NotImplementedError
+
+    def to_json(self) -> str:
+        raise NotImplementedError
 
 
 class LCRoi(Roi):
@@ -379,7 +391,7 @@ class LCRoi(Roi):
         tidyms.peaks.detect_peaks : peak detection of 1D signals.
 
         """
-        self.fill_nan(0.0)
+        self.fill_nan(fill_value="extrapolate")
         noise = peaks.estimate_noise(self.spint)
 
         if smoothing_strength is None:
@@ -390,10 +402,8 @@ class LCRoi(Roi):
         if store_smoothed:
             self.spint = x
 
-        baseline = peaks.estimate_baseline(self.spint, noise)
-        start, apex, end = peaks.detect_peaks(
-            self.spint, noise, baseline, **kwargs
-        )
+        baseline = peaks.estimate_baseline(x, noise)
+        start, apex, end = peaks.detect_peaks(x, noise, baseline, **kwargs)
         features = [Peak(s, a, e) for s, a, e in zip(start, apex, end)]
 
         self.features = features
@@ -403,27 +413,27 @@ class LCRoi(Roi):
 
     def plot(
         self,
-        figure: Optional[bokeh.plotting.Figure] = None,
+        figure: Optional[bokeh.plotting.figure] = None,
         show: bool = True
-    ) -> bokeh.plotting.Figure:     # pragma: no cover
+    ) -> bokeh.plotting.figure:     # pragma: no cover
         """
         Plot the ROI.
 
         Parameters
         ----------
-        figure : bokeh.plotting.Figure or None, default=None
+        figure : bokeh.plotting.figure or None, default=None
             Figure to add the plot. If None, a new figure is created.
         show : bool, default=True
             If True calls ``bokeh.plotting.show`` on the Figure.
 
         Returns
         -------
-        bokeh.plotting.Figure
+        bokeh.plotting.figure
 
         """
         if figure is None:
             fig_params = _plot_bokeh.get_chromatogram_figure_params()
-            figure = bokeh.plotting.Figure(**fig_params)
+            figure = bokeh.plotting.figure(**fig_params)
 
         _plot_bokeh.add_line(figure, self.time, self.spint)
         if self.features:
@@ -453,6 +463,62 @@ class LCRoi(Roi):
             filters = {"width": (4, 60), "snr": (5, None)}
         return filters
 
+    @staticmethod
+    def from_json(s: str):
+        """
+        Creates a LCRoi from a JSON str. Used to serialize LCRoi objects.
+
+        Parameters
+        ----------
+        s : str
+
+        Returns
+        -------
+        LCRoi
+
+        """
+        d = json.loads(s)
+        roi = LCRoi(
+            str_to_array1d(d["spint"]),
+            str_to_array1d(d["mz"]),
+            str_to_array1d(d["time"]),
+            str_to_array1d(d["scan"]),
+        )
+        if "noise" in d:
+            roi.noise = str_to_array1d(d["noise"])
+
+        if "baseline" in d:
+            roi.baseline = str_to_array1d(d["baseline"])
+
+        if "features" in d:
+            roi.features = [Peak(x, y, z) for (x, y, z) in d["features"]]
+        return roi
+
+    def to_json(self) -> str:
+        """
+        Serializes the LCRoi into a JSON str.
+
+        Returns
+        -------
+        str
+
+        """
+        d = dict()
+        d["time"] = array1d_to_str(self.time)
+        d["spint"] = array1d_to_str(self.spint)
+        d["scan"] = array1d_to_str(self.scan)
+        d["mz"] = array1d_to_str(self.mz)
+        if self.features is not None:
+            d["features"] = [[x.start, x.apex, x.end] for x in self.features]
+
+        if self.baseline is not None:
+            d["baseline"] = array1d_to_str(self.baseline)
+
+        if self.noise is not None:
+            d["noise"] = array1d_to_str(self.noise)
+        d_json = json.dumps(d)
+        return d_json
+
 
 class Chromatogram(LCRoi):
     """
@@ -506,8 +572,8 @@ class Feature:
             msg = "start must be lower than end"
             raise ValueError(msg)
 
-        self.start = start
-        self.end = end
+        self.start = int(start)
+        self.end = int(end)
 
     def __repr__(self):
         str_repr = "{}(start={}, end={})"
@@ -518,7 +584,7 @@ class Feature:
     def plot(
         self,
         roi: Roi,
-        figure: bokeh.plotting.Figure,
+        figure: bokeh.plotting.figure,
         color: str,
         **varea_params
     ):
@@ -555,7 +621,7 @@ class Peak(Feature):
         except AssertionError:
             msg = "start must be lower than loc and loc must be lower than end"
             raise InvalidPeakException(msg)
-        self.apex = apex
+        self.apex = int(apex)
 
     def __repr__(self):
         str_repr = "{}(start={}, apex={}, end={})"
@@ -563,6 +629,34 @@ class Peak(Feature):
         str_repr = str_repr.format(name, self.start, self.apex, self.end)
         return str_repr
 
+    def get_rt_start(self, roi: LCRoi) -> float:
+        """
+        Computes the start of the peak, in time units
+        Parameters
+        ----------
+        roi : LCRoi
+
+        Returns
+        -------
+        float
+
+        """
+        return roi.time[self.start]
+
+    def get_rt_end(self, roi: LCRoi) -> float:
+        """
+        Computes the end of the peak, in time units
+        Parameters
+        ----------
+        roi : LCRoi
+
+        Returns
+        -------
+        float
+
+        """
+        return roi.time[self.end - 1]
+
     def get_rt(self, roi: LCRoi) -> float:
         """
         Finds the peak location in the ROI rt, using spint as weights.
@@ -578,7 +672,9 @@ class Peak(Feature):
 
         """
         weights = roi.spint[self.start:self.end]
-        weights[weights < 0] = 0
+        if roi.baseline is not None:
+            weights = weights - roi.baseline[self.start:self.end]
+        weights = np.maximum(weights, 0)
         loc = np.abs(np.average(roi.time[self.start:self.end], weights=weights))
         return loc
 
@@ -755,7 +851,9 @@ class Peak(Feature):
             c.WIDTH: self.get_width(roi),
             c.SNR: self.get_snr(roi),
             c.MZ: self.get_mean_mz(roi),
-            c.MZ_STD: self.get_mz_std(roi)
+            c.MZ_STD: self.get_mz_std(roi),
+            c.RT_START: self.get_rt_start(roi),
+            c.RT_END: self.get_rt_end(roi)
         }
         return descriptors
 
